local AddonName, GGL = ...
--[[-----------------------------------------------------------------------------
Label Widget
Displays text and optionally an icon.
-------------------------------------------------------------------------------]]
local Type, Version = "GoldGoblinLootGUIWidget-InviteLine", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

local L = GGL.Locale

-- Lua APIs
local max, select, pairs = math.max, select, pairs
local tonumber = tonumber
local abs, floor = math.abs, math.floor
local format = string.format

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent
local GetItemIcon, GetItemInfo = GetItemIcon, GetItemInfo
local UnitFactionGroup = UnitFactionGroup

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]
local GAP_SIZE = 2
local FIX_HEIGHT = 14

local STATE_ID_LIST = GGL.INVITE_KEY_LIST
local STATE_LIST_ATLAS_ICONS = GGL.INVITE_KEY_LIST_ATLAS_ICONS

local function Frame_OnEnter(self)
    --[[
    if not self.obj.itemID then return end
    local tooltip = _G.GameTooltip
    tooltip:ClearLines()
    GGL.GUI:SetToolTipPoint(tooltip, self.obj.ttParent, self.obj.ttPoint, 2, 0)
    tooltip:SetItemByID(self.obj.itemID)
    tooltip:Show()
    ]]--
end

local function Frame_OnLeave(self)
    --_G.GameTooltip:Hide()
end

local function ButtonRight_OnCLick(frame, ...)
	AceGUI:ClearFocus()
	PlaySound(852) -- SOUNDKIT.IG_MAINMENU_OPTION
	frame.obj:Fire("DeleteOnClick", ...)
end

local function ButtonRight2_OnCLick(frame, ...)
	AceGUI:ClearFocus()
	PlaySound(852) -- SOUNDKIT.IG_MAINMENU_OPTION
	frame.obj:Fire("StateOnClick", ...)
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
    ["OnAcquire"] = function(self)
        self:SetFaction()
        self:SetState()
        self:SetCount()
        self:ResetToolTipPoint()
    end,

    ["OnRelease"] = function(self)
        self:SetPlayer(nil)
    end,

    ["OnWidthSet"] = function(self, width)

    end,

    ["SetToolTipPoint"] = function(self, ttPoint, parent)
        self.ttPoint = ttPoint or "BOTTOM_X"
        self.ttParent = parent or self.frame
    end,

    ["ResetToolTipPoint"] = function(self)
        self.ttPoint = "BOTTOM_X"
        self.ttParent = self.frame
    end,

    ["SetName"] = function(self, name)
        name = name or ""
        self.text1:SetText(name)
    end,

    ["SetPlayer"] = function(self, player)
        if self.player and self.player == player then return end
        if self.player then
            self.player:UnregisterCallbackEvent("DisplayNameUpdate", self.__onDisplayNameUpdate)
            self.player = nil
        end
        if player then
            self:SetName(player.displayName)
            self:SetFaction(player.factionName)
            player:RegisterCallbackEvent("DisplayNameUpdate", self.__onDisplayNameUpdate)
        end
        self.player = player
    end,

    ["GetPlayer"] = function(self)
        return self.player
    end,

    -- englishFaction = UnitFactionGroup("player")
    ["SetFaction"] = function(self, englishFaction)
        if englishFaction == "player" then
            englishFaction = UnitFactionGroup("player")
        end
        if englishFaction == "Horde" then
            self.imageL:SetAtlas("communities-create-button-wow-horde")
        elseif englishFaction == "Alliance" then
            self.imageL:SetAtlas("communities-create-button-wow-alliance")
        elseif englishFaction == "Neutral" then
            self.imageL:SetAtlas("AlliedRace-Crest-Nightborne")
        else
            return self.imageL:Hide()
        end
        self.imageL:Show()
    end,

    ["SetState"] = function(self, state)
        if state and STATE_LIST_ATLAS_ICONS[state] then
            self.buttonRight2Image:SetAtlas(STATE_LIST_ATLAS_ICONS[state])
            self.buttonRight2:Show()
        else
            self.buttonRight2:Hide()
        end
    end,

    ["SetCount"] = function(self, count)
        if count and (count % 2 == 0) then
            self.frameBG:Show()
        else
            self.frameBG:Hide()
        end
    end
}


--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
    local widget = {}
    widget.type  = Type

    local frame = CreateFrame("Frame", nil, UIParent)
    frame:SetHeight(FIX_HEIGHT)
    frame:SetScript("OnEnter", Frame_OnEnter)
    frame:SetScript("OnLeave", Frame_OnLeave)
    frame.obj = widget
    widget.frame = frame

    local frameBG = frame:CreateTexture(nil, "BACKGROUND")
    frameBG:SetAllPoints(frame)
    frameBG:SetColorTexture(1, 1, 1, 0.2)
    frameBG:Hide()
    widget.frameBG = frameBG

    local imageL = frame:CreateTexture(nil, "BACKGROUND")
    imageL:SetPoint("LEFT", frame, "LEFT", 0, 0)
    imageL:SetWidth(FIX_HEIGHT-2)
    imageL:SetHeight(FIX_HEIGHT-2)
    imageL:SetAtlas("groupfinder-icon-leader")
    widget.imageL = imageL

    local buttonRight = CreateFrame("Button", nil, frame)
    buttonRight:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
    buttonRight:SetWidth(FIX_HEIGHT-2)
    buttonRight:SetHeight(FIX_HEIGHT-2)
    buttonRight:SetScript("OnClick", ButtonRight_OnCLick)
    buttonRight.obj = widget
    widget.buttonRight = buttonRight

    local buttonRightImage = buttonRight:CreateTexture(nil, "BACKGROUND")
    buttonRightImage:SetAllPoints(buttonRight)
    buttonRightImage:SetAtlas("transmog-icon-remove")
    widget.buttonRightImage = buttonRightImage

    local buttonRight2 = CreateFrame("Button", nil, frame)
    buttonRight2:SetPoint("RIGHT", buttonRight, "LEFT", -GAP_SIZE, 0)
    buttonRight2:Hide()
    buttonRight2:SetWidth(FIX_HEIGHT-2)
    buttonRight2:SetHeight(FIX_HEIGHT-2)
    buttonRight2:SetScript("OnClick", ButtonRight2_OnCLick)
    buttonRight2.obj = widget
    widget.buttonRight2 = buttonRight2

    local buttonRight2Image = buttonRight2:CreateTexture(nil, "BACKGROUND")
    buttonRight2Image:SetAllPoints(buttonRight2)
    buttonRight2Image:SetAtlas("groupfinder-waitdot")
    widget.buttonRight2Image = buttonRight2Image

    local text1 = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    text1:SetPoint("LEFT", imageL, "RIGHT", GAP_SIZE, 0)
    text1:SetPoint("RIGHT", buttonRight2, "LEFT", -GAP_SIZE, 0)
    text1:SetJustifyH("LEFT")
    text1:SetJustifyV("MIDDLE")
    text1:SetWordWrap(false)
    text1:SetHeight(FIX_HEIGHT)
    widget.text1 = text1

    -- methodes
    for method, func in pairs(methods) do
        widget[method] = func
    end

    -- cb
    widget.__onDisplayNameUpdate = function(self, displayName)
        widget:SetName(displayName)
    end

    return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
