local AddonName, GGL = ...

local AceGUI = LibStub("AceGUI-3.0")
local GGL_Templates = GGL.Templates
local L = GGL.Locale

local Statistics = {}
GGL.GUI.Statistics = Statistics

-- lua

-- wow

-- GGL
local GetMoneyText = GGL.GetMoneyText
local GetMoneyTextShort = GGL.GetMoneyTextShort

-- ##################
-- GUI - shared
-- ##################
local ExportFrame

local Templates
do
    Templates = {
        ["ShowExportFrame"] = function(data, exportType)
            if not ExportFrame then
                local frame = AceGUI:Create("Frame")
                frame:SetTitle(L["Export"])
                --frame:SetStatusText(listName)
                frame:SetLayout("Fill")

                local multiEditbox = AceGUI:Create("MultiLineEditBox")
                multiEditbox.frame:SetClipsChildren(true)
                multiEditbox.editBox:SetMaxBytes(nil)
                multiEditbox.editBox:SetScript("OnEscapePressed", function()
                    frame:Close()
                end)
                multiEditbox.editBox:SetScript("OnMouseUp", function()
                    multiEditbox.editBox:HighlightText()
                end)
                multiEditbox.editBox:SetScript("OnChar", function(self)
                    multiEditbox:SetText(self.exportText)
                    multiEditbox.editBox:HighlightText()
                end)
                frame.multiEditbox = multiEditbox
                frame:AddChild(multiEditbox)

                ExportFrame = frame
            end

            local multiEditbox = ExportFrame.multiEditbox

            local exportText
            if exportType == "csv" then
                exportText = GGL.Export:ExportAsCSV(data, ",", true)
            end

            multiEditbox.editBox.exportText = exportText or ""
            multiEditbox:SetText(exportText or "")
            ExportFrame:Show()

            multiEditbox.editBox:HighlightText()
            multiEditbox.editBox:SetFocus()
            return ExportFrame
        end,

        ["GetItemQualityBox"] = function(content)
            local qualityGroup = AceGUI:Create("SimpleGroup")
            qualityGroup:ClearAllPoints()
            qualityGroup.frame:SetParent(content)
            qualityGroup:SetLayout("List")
            qualityGroup.barList = {}

            for quality = (#GGL.ITEM_QUALITY_INFO-1), 0, -1 do
                local qData = GGL.ITEM_QUALITY_INFO[quality]

                local bar = AceGUI:Create("GoldGoblinLootGUIWidget-ProgressBar")
                bar:SetFullWidth(true)
                bar:SetBarColor(qData.color.r, qData.color.g, qData.color.b)
                bar:SetPercent(true,true)

                qualityGroup:AddChild(bar)
                qualityGroup.barList[quality] = bar
            end

            return qualityGroup
        end,

        ["GetBoxWithHeader"] = function(parent)
            local frame = GGL.GUI:CreateBoxFrame(parent)
            frame:SetSize(200,100)

            local text = frame:CreateFontString(nil, "BORDER", "GameFontNormal", 2)
            text:SetFontObject(GGL.GetFont(12, "THICKOUTLINE"))
            text:SetPoint("CENTER", frame, "TOP", 0, -2)
            text:SetJustifyH("CENTER")
            text:SetJustifyV("MIDDLE")
            text:SetHeight(12)
            text:SetWordWrap(false)
            text:SetNonSpaceWrap(false)

            local content = CreateFrame("FRAME", nil, frame)
            content:SetPoint("TOPLEFT", frame, 8, -13)
            content:SetPoint("BOTTOMRIGHT", frame, -8, 4)

            -- header fix 5
            -- +17

            frame.text = text
            frame.content = content

            return frame
        end,
    }
end
GGL.GUI.Statistics.Templates = Templates

-- ##################
-- Base
-- ##################

-- ##################
-- GUI - QuickInfo
-- ##################
local QuickInfoFrame
local TOP_ITEMS_LIST_SIZE = 5
local PLAYER_PLAYING_FORMAT, PLAYER_PLAYING_FORMAT_COLOR = "Playing %s-%s", "Playing |c%s%s|r-%s"
local ITEM_NUM_FORMAT = "%s x %d"
local QUALITY_TEXT_FORMAT = "%d - %d%%"

local QUICK_INFO_FRAME_PROTO = {
    ["Clear"] = function(self)
        self:SetStatistics()
        self:SetPlayerTemplate()
        self:SetTopItems()
        self:SetItemQualityList()

        -- frames clear
        self:UpdateTextFromStatistics()
    end,

    --- Sets the statistics for the frame
    ["SetStatistics"] = function(self, statistics)
        self.statistics = statistics
        self:UpdateFromStatistics()
    end,

    --- Updates the gui with info from a statistics register
    -- Needs a player template registered
    ["UpdateFromStatistics"] = function(self, playerTemplate, statistics, addPlayerIfMissing)
        if playerTemplate then self:SetPlayerTemplate(playerTemplate) end
        if statistics then return self:SetStatistics(statistics) end
        if not self.statistics then return end
        if not self.playerTemplate then return end

        local player = self.statistics:GetPlayerTable(self.playerTemplate, nil, not addPlayerIfMissing)
        if not player then return end

        self:SetItemQualityList(player.itemQualityCount, player.numItems)
        self:SetTopItems(self.statistics:GetTopListForMergeID(self.playerTemplate))

        -- set raw gold
        self:UpdateTextFromStatistics()
    end,

    ["UpdateTextFromStatistics"] = function(self)
        local frame = QuickInfoFrame.frame

        if self.statistics then
            local statistics = self.statistics
            -- gold
            frame.goldBox.rawGold:SetText(GetMoneyTextShort(statistics:GetRawGoldForMergeID(self.playerTemplate)))
            frame.goldBox.itemValue:SetText(GetMoneyTextShort(statistics:GetItemValueGoldForMergeID(self.playerTemplate)))
            -- player
            local numEntered, numReset = statistics:GetInstanceResetDataForMergeID(self.playerTemplate)
            frame.iniBox.iniEntered:SetText(numEntered or 0)
            frame.iniBox.iniReset:SetText(numReset or 0)
            frame.iniBox.usedChars:SetText(statistics:GetNumCharsForMergeID(self.playerTemplate))
        else
            -- gold
            frame.goldBox.rawGold:SetText(GetMoneyTextShort(0))
            frame.goldBox.itemValue:SetText(GetMoneyTextShort(0))
            -- player
            frame.iniBox.iniEntered:SetText(0)
            frame.iniBox.iniReset:SetText(0)
            frame.iniBox.usedChars:SetText(0)
        end
        --GetNumCharsForMergeID
    end,

    --- Sets a player template for live updates
    -- @param       playerTemplate      <playerTemplate>
    ["SetPlayerTemplate"] = function(self, playerTemplate)
        if self.playerTemplate then
            self.playerTemplate:UnregisterCallbackEvent("PlayerUpdate", self.CB_OnPlayerUpdate)
            self.playerTemplate = nil
        end
        if not playerTemplate then
            return
        end


        self.playerTemplate = playerTemplate
        playerTemplate:RegisterCallbackEvent("PlayerUpdate", self.CB_OnPlayerUpdate)

        local bTagBox = QuickInfoFrame.frame.bTagBox

        -- set battleTag
        if playerTemplate.bnPlayer and playerTemplate.bnPlayer.battleTag then
            bTagBox.text:SetText(playerTemplate.bnPlayer.battleTag)
        else
            bTagBox.text:SetText("")
        end

        -- set name
        if playerTemplate.className then
            bTagBox.name:SetText(format(PLAYER_PLAYING_FORMAT_COLOR, GGL.CLASS_COLORS[GGL.LOCALIZED_CLASS_TO_NAME[playerTemplate.className]].colorStr, playerTemplate.characterName, playerTemplate.realmName))
        else
            bTagBox.name:SetText(format(PLAYER_PLAYING_FORMAT, playerTemplate.characterName, playerTemplate.characterName))
        end

        -- set faction icon
        if playerTemplate.factionName and GGL.FACTION_ICON_ATLAS[playerTemplate.factionName] then
            bTagBox.factionIcon:SetAtlas(GGL.FACTION_ICON_ATLAS[playerTemplate.factionName])
            bTagBox.factionIcon:Show()
        else
            bTagBox.factionIcon:Hide()
        end

        -- set ggl version
        if playerTemplate.gglVersion then
            bTagBox.version:SetText(playerTemplate.gglVersion)
            bTagBox.version:Show()
        else
            bTagBox.version:Hide()
        end

        self.CB_OnPlayerUpdate(playerTemplate)
    end,

    --- Sets the top item
    -- @param       topItems        <table> {itemID, value}
    ["SetTopItems"] = function(self, topItems)
        local buttons = QuickInfoFrame.frame.topItemsBox.buttons

        for i = 1, #buttons do
            local button, item = buttons[i], topItems and topItems[i] or nil
            if topItems and item then
                local itemName, itemLink = GetItemInfo(item[1])

                button.icon:SetTexture(GetItemIcon(item[1]))
                button.itemName:SetFormattedText(ITEM_NUM_FORMAT, itemLink or "", self.statistics:GetItemCountForMergeID(self.playerTemplate, item[1]))
                button.itemValue:SetText(GetMoneyTextShort(item[2]))

                button.itemID = item[1]

                button:Show()
            else
                button.itemID = nil
                button:Hide()
            end
        end
    end,

    --- Sets item quality count
    -- @param       qualityList        <table> [qualityID] = {count, percent}
    ["SetItemQualityList"] = function(self, qualityList, numItems)
        local qFrames = QuickInfoFrame.frame.itemQualityBox.qFrames

        for i = 0, #qFrames do
            if qFrames[i] then
                if qualityList and qualityList[i] and qualityList[i] > 0 and numItems > 0 then
                    qFrames[i].text:SetFormattedText(QUALITY_TEXT_FORMAT, qualityList[i] or 0, (100/(numItems))*qualityList[i])
                else
                    qFrames[i].text:SetFormattedText(QUALITY_TEXT_FORMAT, 0, 0)
                end
            end
        end
    end,


    -- CB's
    ["CB_OnPlayerUpdate"] = function(playerTemplate)
        local bTagBox = QuickInfoFrame.frame.bTagBox

        -- set zone
        if playerTemplate.areaName then
            bTagBox.zone:SetText(playerTemplate.areaName)
        else
            bTagBox.zone:SetText(UNKNOWN)
        end
    end,
}

local function QuickInfoFrame_Item_OnEnter(self)
    if not self.itemID then return end
    local tooltip = _G.GameTooltip
    tooltip:ClearLines()
    tooltip:SetOwner(UIParent,"ANCHOR_CURSOR")
    tooltip:SetItemByID(self.itemID)
    tooltip:Show()
end

local function QuickInfoFrame_Item_OnLeave(self)
    _G.GameTooltip:Hide()
end

local function QuickInfoFrame_AddItemButton(parent)
    local height = 12
    local frame = CreateFrame("Frame", nil, parent)
    frame:SetScript("OnEnter", QuickInfoFrame_Item_OnEnter)
    frame:SetScript("OnLeave", QuickInfoFrame_Item_OnLeave)
    frame:SetHeight(height)

    local icon = frame:CreateTexture(nil, "BACKGROUND")
    icon:SetPoint("LEFT", frame, "LEFT", 0, 0)
    icon:SetWidth(height)
    icon:SetHeight(height)
    frame.icon = icon

    local itemName = frame:CreateFontString(nil, "BACKGROUND")
    itemName:SetPoint("LEFT", icon, "RIGHT", 1, 0)
    itemName:SetFontObject(GGL.GetFont(height))
    itemName:SetPoint("RIGHT", frame, 0, 0)
    itemName:SetJustifyH("LEFT")
    itemName:SetJustifyV("MIDDLE")
    itemName:SetWordWrap(false)
    itemName:SetHeight(height)
    frame.itemName = itemName

    local itemValue = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    itemValue:SetAllPoints(itemName)
    itemValue:SetFontObject(GGL.GetFont(height))
    itemValue:SetJustifyH("RIGHT")
    itemValue:SetJustifyV("MIDDLE")
    itemValue:SetWordWrap(false)
    itemValue:SetHeight(height)
    frame.itemValue = itemValue

    return frame
end


local function CreateQuickInfoFrame()
    if not QuickInfoFrame then
        local fullFrameHeight = 20

        local frame = GGL.GUI:CreateBoxFrame(UIParent)
        frame:SetPoint("CENTER", UIParent, 100, 0)
        frame:SetWidth(270)
        frame:SetHeight(350)
        frame:SetBackdropColor(0, 0, 0, 0.7)
        frame:SetBackdropBorderColor(0, 0, 0, 1)

        local content = CreateFrame("FRAME", nil, frame)
        content:SetPoint("TOPLEFT", frame, 10, -10)
        content:SetPoint("BOTTOMRIGHT", frame, -10, 10)
        frame.content = content

        -- #######
        -- battleTag
        local bTagBox = Templates.GetBoxWithHeader(content)
        bTagBox.text:SetText(L["Battle.Tag"])
        bTagBox:SetPoint("TOPLEFT", content, 0, -5)
        bTagBox:SetPoint("RIGHT", content, 0, 0)
        bTagBox:SetHeight(48)
        frame.bTagBox = bTagBox

        bTagBox.factionIcon = bTagBox.content:CreateTexture(nil, "BACKGROUND")
        bTagBox.factionIcon:SetPoint("RIGHT", bTagBox, -1, 0)
        bTagBox.factionIcon:SetWidth(bTagBox:GetHeight()-2)
        bTagBox.factionIcon:SetHeight(bTagBox:GetHeight()-2)
        bTagBox.factionIcon:SetAlpha(0.25)

        bTagBox.version = bTagBox.content:CreateFontString(nil, "BORDER", nil, 2)
        bTagBox.version:SetFontObject(GGL.GetFont(12))
        bTagBox.version:SetHeight(12)
        bTagBox.version:SetPoint("TOPRIGHT", bTagBox, -2, -2)
        bTagBox.version:SetPoint("LEFT", bTagBox, 0, 0)
        bTagBox.version:SetJustifyH("RIGHT")
        bTagBox.version:SetJustifyV("MIDDLE")
        bTagBox.version:SetAlpha(0.5)
        bTagBox.version:SetText(UNKNOWN)

        bTagBox.name = bTagBox.content:CreateFontString(nil, "BORDER", nil, 2)
        bTagBox.name:SetFontObject(GGL.GetFont(12))
        bTagBox.name:SetHeight(12)
        bTagBox.name:SetPoint("TOPLEFT", bTagBox.content, 0, 0)
        bTagBox.name:SetPoint("RIGHT", bTagBox.content, 0, 0)
        bTagBox.name:SetJustifyH("LEFT")
        bTagBox.name:SetJustifyV("MIDDLE")
        bTagBox.name:SetText(UNKNOWN)

        bTagBox.zoneText = bTagBox.content:CreateFontString(nil, "BORDER", nil, 2)
        bTagBox.zoneText:SetFontObject(GGL.GetFont(16))
        bTagBox.zoneText:SetHeight(16)
        bTagBox.zoneText:SetPoint("TOPLEFT", bTagBox.name, "BOTTOMLEFT", 0, -2)
        bTagBox.zoneText:SetJustifyH("LEFT")
        bTagBox.zoneText:SetJustifyV("MIDDLE")
        bTagBox.zoneText:SetText(L["Zone:"])
        bTagBox.zoneText:SetWidth(bTagBox.zoneText:GetUnboundedStringWidth()+2)

        bTagBox.zone = bTagBox.content:CreateFontString(nil, "BORDER", "GameFontNormal", 2)
        bTagBox.zone:SetFontObject(GGL.GetFont(16))
        bTagBox.zone:SetHeight(16)
        bTagBox.zone:SetPoint("LEFT", bTagBox.zoneText, "RIGHT", 0, 0)
        bTagBox.zone:SetPoint("RIGHT", bTagBox.content, 0, 0)
        bTagBox.zone:SetJustifyH("LEFT")
        bTagBox.zone:SetJustifyV("MIDDLE")
        bTagBox.zone:SetText(UNKNOWN)

        fullFrameHeight = fullFrameHeight + 5 + bTagBox:GetHeight()
        -- battleTag
        -- #######

        -- #######
        -- left > reset box
        local iniBox = GGL.GUI:CreateBoxFrame(content)
        iniBox:SetHeight(66) -- 58
        iniBox:SetPoint("TOPLEFT", bTagBox, "BOTTOMLEFT", 0, -5)
        iniBox:SetPoint("RIGHT", content, "CENTER", -2.5, 0)
        frame.iniBox = iniBox

        iniBox.iniEntered = iniBox:CreateFontString(nil, "BORDER", nil, 2)
        iniBox.iniEntered:SetFontObject(GGL.GetFont(16))
        iniBox.iniEntered:SetHeight(16)
        iniBox.iniEntered:SetPoint("TOPRIGHT", iniBox, -2, -6)
        iniBox.iniEntered:SetJustifyH("CENTER")
        iniBox.iniEntered:SetJustifyV("MIDDLE")
        iniBox.iniEntered:SetText(999)
        iniBox.iniEntered:SetWidth(iniBox.iniEntered:GetUnboundedStringWidth()+5)

        iniBox.iniEnteredDesc = iniBox:CreateFontString(nil, "BORDER", nil, 2)
        iniBox.iniEnteredDesc:SetFontObject(GGL.GetFont(12))
        iniBox.iniEnteredDesc:SetHeight(12)
        iniBox.iniEnteredDesc:SetPoint("TOPRIGHT", iniBox.iniEntered, "TOPLEFT", 0, -2)
        iniBox.iniEnteredDesc:SetPoint("LEFT", iniBox, 8, 0)
        iniBox.iniEnteredDesc:SetJustifyH("LEFT")
        iniBox.iniEnteredDesc:SetJustifyV("MIDDLE")
        iniBox.iniEnteredDesc:SetTextColor(0.625, 0.625, 0.625)
        iniBox.iniEnteredDesc:SetText(L["Instances:"])

        iniBox.iniReset = iniBox:CreateFontString(nil, "BORDER", nil, 2)
        iniBox.iniReset:SetFontObject(GGL.GetFont(14))
        iniBox.iniReset:SetHeight(14)
        iniBox.iniReset:SetWidth(iniBox.iniEntered:GetWidth())
        iniBox.iniReset:SetPoint("TOPRIGHT", iniBox.iniEntered, "BOTTOMRIGHT", 0, -5)
        iniBox.iniReset:SetJustifyH("CENTER")
        iniBox.iniReset:SetJustifyV("MIDDLE")
        iniBox.iniReset:SetText(15)

        iniBox.iniResetDesc = iniBox:CreateFontString(nil, "BORDER", nil, 2)
        iniBox.iniResetDesc:SetFontObject(GGL.GetFont(12))
        iniBox.iniResetDesc:SetHeight(12)
        iniBox.iniResetDesc:SetPoint("TOPRIGHT", iniBox.iniReset, "TOPLEFT", 0, -1)
        iniBox.iniResetDesc:SetPoint("LEFT", iniBox.iniEnteredDesc, 0, 0)
        iniBox.iniResetDesc:SetJustifyH("LEFT")
        iniBox.iniResetDesc:SetJustifyV("MIDDLE")
        iniBox.iniResetDesc:SetTextColor(0.625, 0.625, 0.625)
        iniBox.iniResetDesc:SetText(L["Instance Resets:"])

        iniBox.usedChars = iniBox:CreateFontString(nil, "BORDER", nil, 2)
        iniBox.usedChars:SetFontObject(GGL.GetFont(14))
        iniBox.usedChars:SetHeight(14)
        iniBox.usedChars:SetWidth(iniBox.iniEntered:GetWidth())
        iniBox.usedChars:SetPoint("TOPRIGHT", iniBox.iniReset, "BOTTOMRIGHT", 0, -5)
        iniBox.usedChars:SetJustifyH("CENTER")
        iniBox.usedChars:SetJustifyV("MIDDLE")
        iniBox.usedChars:SetText(15)

        iniBox.usedCharsDesc = iniBox:CreateFontString(nil, "BORDER", nil, 2)
        iniBox.usedCharsDesc:SetFontObject(GGL.GetFont(12))
        iniBox.usedCharsDesc:SetHeight(12)
        iniBox.usedCharsDesc:SetPoint("TOPRIGHT", iniBox.usedChars, "TOPLEFT", 0, -1)
        iniBox.usedCharsDesc:SetPoint("LEFT", iniBox.iniEnteredDesc, 0, 0)
        iniBox.usedCharsDesc:SetJustifyH("LEFT")
        iniBox.usedCharsDesc:SetJustifyV("MIDDLE")
        iniBox.usedCharsDesc:SetTextColor(0.625, 0.625, 0.625)
        iniBox.usedCharsDesc:SetText(L["Characters:"])
        -- left > reset box end
        -- #######


        -- #######
        -- right > reset box end
        local goldBoxSingleHeight = (iniBox:GetHeight() - 2 ) / 4
        local goldBox = GGL.GUI:CreateBoxFrame(content)
        goldBox:SetPoint("TOPRIGHT", bTagBox, "BOTTOMRIGHT", 0, -5)
        goldBox:SetPoint("BOTTOMLEFT", iniBox, "BOTTOMRIGHT", 5, 0)
        frame.goldBox = goldBox

        goldBox.rawGoldDesc = goldBox:CreateFontString(nil, "BORDER", nil, 2)
        goldBox.rawGoldDesc:SetFontObject(GGL.GetFont(12))
        goldBox.rawGoldDesc:SetHeight(goldBoxSingleHeight)
        goldBox.rawGoldDesc:SetPoint("TOPLEFT", goldBox, 1, -1)
        goldBox.rawGoldDesc:SetPoint("TOPRIGHT", goldBox, -1, -1)
        goldBox.rawGoldDesc:SetJustifyH("CENTER")
        goldBox.rawGoldDesc:SetJustifyV("MIDDLE")
        goldBox.rawGoldDesc:SetTextColor(0.625, 0.625, 0.625)
        goldBox.rawGoldDesc:SetText(L["Raw gold"])

        goldBox.rawGold = goldBox:CreateFontString(nil, "BORDER", nil, 2)
        goldBox.rawGold:SetFontObject(GGL.GetFont(14))
        goldBox.rawGold:SetHeight(goldBoxSingleHeight)
        goldBox.rawGold:SetPoint("TOPLEFT", goldBox.rawGoldDesc, "BOTTOMLEFT", 0, 0)
        goldBox.rawGold:SetPoint("TOPRIGHT", goldBox.rawGoldDesc, "BOTTOMRIGHT", 0, 0)
        goldBox.rawGold:SetJustifyH("CENTER")
        goldBox.rawGold:SetJustifyV("MIDDLE")
        goldBox.rawGold:SetText(GetMoneyTextShort(1234567899999))

        goldBox.itemValueDesc = goldBox:CreateFontString(nil, "BORDER", nil, 2)
        goldBox.itemValueDesc:SetFontObject(GGL.GetFont(12))
        goldBox.itemValueDesc:SetHeight(goldBoxSingleHeight)
        goldBox.itemValueDesc:SetPoint("TOPLEFT", goldBox.rawGold, "BOTTOMLEFT", 0, 0)
        goldBox.itemValueDesc:SetPoint("TOPRIGHT", goldBox.rawGold, "BOTTOMRIGHT", 0, 0)
        goldBox.itemValueDesc:SetJustifyH("CENTER")
        goldBox.itemValueDesc:SetJustifyV("MIDDLE")
        goldBox.itemValueDesc:SetTextColor(0.625, 0.625, 0.625)
        goldBox.itemValueDesc:SetText(L["Item value"])

        goldBox.itemValue = goldBox:CreateFontString(nil, "BORDER", nil, 2)
        goldBox.itemValue:SetFontObject(GGL.GetFont(14))
        goldBox.itemValue:SetHeight(goldBoxSingleHeight)
        goldBox.itemValue:SetPoint("TOPLEFT", goldBox.itemValueDesc, "BOTTOMLEFT", 0, 0)
        goldBox.itemValue:SetPoint("TOPRIGHT", goldBox.itemValueDesc, "BOTTOMRIGHT", 0, 0)
        goldBox.itemValue:SetJustifyH("CENTER")
        goldBox.itemValue:SetJustifyV("MIDDLE")
        goldBox.itemValue:SetText(GetMoneyTextShort(1234567899999))
        -- right > reset box end
        -- #######

        fullFrameHeight = fullFrameHeight + 5 + iniBox:GetHeight()

        -- #######
        -- topItems box
        local topItemsBox = Templates.GetBoxWithHeader(content)
        topItemsBox.text:SetText(L["TopItems"])
        topItemsBox:SetPoint("TOP", iniBox, "BOTTOM", 0, -10)
        topItemsBox:SetPoint("LEFT", content, 0, 0)
        topItemsBox:SetPoint("RIGHT", content, 0, 0)
        topItemsBox:SetHeight(18 + (TOP_ITEMS_LIST_SIZE*12) + ((TOP_ITEMS_LIST_SIZE-1)*2))
        frame.topItemsBox = topItemsBox

        topItemsBox.buttons = {}
        for i = 1, TOP_ITEMS_LIST_SIZE do
            local button = QuickInfoFrame_AddItemButton(topItemsBox.content)

            if i == 1 then
                button:SetPoint("TOPLEFT", topItemsBox.content, 0, 0)
            else
                button:SetPoint("TOPLEFT", topItemsBox.buttons[i-1], "BOTTOMLEFT", 0, -2)
            end
            button:SetPoint("RIGHT", topItemsBox.content, 0, 0)

            topItemsBox.buttons[i] = button
        end

        fullFrameHeight = fullFrameHeight + 10 + topItemsBox:GetHeight()
        -- topItems box
        -- #######

        -- #######
        -- topItems box
        local itemQualityBox = GGL.GUI:CreateBoxFrame(content)
        itemQualityBox:SetPoint("TOPLEFT", topItemsBox, "BOTTOMLEFT", 0, -5)
        itemQualityBox:SetPoint("RIGHT", topItemsBox, 0, 0)
        itemQualityBox:SetHeight(28+5+10)
        frame.itemQualityBox = itemQualityBox

        itemQualityBox.qFrames = {}
        local qFrameWidth = (content:GetWidth()-16) / 3
        local qFrameHeight = 14
        for quality = 0, #GGL.ITEM_QUALITY_INFO do
            local qData = GGL.ITEM_QUALITY_INFO[quality]

            local qFrame = CreateFrame("FRAME", nil, itemQualityBox)
            qFrame:SetWidth(qFrameWidth)
            qFrame:SetHeight(qFrameHeight)

            if quality == 0 then
                qFrame:SetPoint("TOPLEFT", itemQualityBox, 8, -5)
            elseif quality == 3 then
                qFrame:SetPoint("TOPLEFT", itemQualityBox.qFrames[0], "BOTTOMLEFT", 0, -5)
            else
                qFrame:SetPoint("LEFT", itemQualityBox.qFrames[quality-1], "RIGHT", 0, 0)
            end

            local icon = qFrame:CreateTexture(nil, "BACKGROUND")
            icon:SetPoint("LEFT", qFrame, 0, 0)
            icon:SetWidth(qFrameHeight)
            icon:SetHeight(qFrameHeight)
            icon:SetColorTexture(qData.color.r, qData.color.g, qData.color.b)
            qFrame.icon = icon

            local text = qFrame:CreateFontString(nil, "BACKGROUND")
            text:SetPoint("LEFT", icon, "RIGHT", 1, 0)
            text:SetFontObject(GGL.GetFont(12))
            text:SetHeight(12)
            text:SetPoint("RIGHT", qFrame, 0, 0)
            text:SetJustifyH("LEFT")
            text:SetJustifyV("MIDDLE")
            text:SetWordWrap(false)
            text:SetFormattedText(QUALITY_TEXT_FORMAT, 0, 0)
            qFrame.text = text

            itemQualityBox.qFrames[quality] = qFrame
        end


        fullFrameHeight = fullFrameHeight + 5 + itemQualityBox:GetHeight()
        -- topItems box
        -- #######

        frame:SetHeight(fullFrameHeight)

        local window = {
            frame = frame,
        }
        setmetatable(window, {__index=QUICK_INFO_FRAME_PROTO})

        QuickInfoFrame = window
    end
    return QuickInfoFrame
end

--- Shows the quickInfoFrame
-- @param       parent          <frame>
-- @param       ...             <points> :SetPoint(...)
function Statistics:ShowQuickInfoFrame(parent, ...)
    local frame = CreateQuickInfoFrame()
    frame:Clear()
    frame.frame:ClearAllPoints()
    frame.frame:SetParent(parent)
    frame.frame:SetPoint(...)
    frame.frame:Show()
    return frame
end

-- ##################
-- GUI
-- ##################
local FrameTypeRegister = {}
local WIDTH_V, HEIGHT_V = 600, 500
local WIDTH_M, HEIGHT_M = 500, 500

local function BaseFrameDropDown_OnValueChanged(self, event, value)
    self = self.obj or self
    local frame = self.frame

    -- create frame if not existing
    if not frame.contentGroupRegister[value] then
        local contentGroup = AceGUI:Create("SimpleGroup")
        contentGroup:ClearAllPoints()
        contentGroup.frame:SetParent(frame.content)
        contentGroup:SetLayout("Flow")
        contentGroup.frame:SetPoint("TOPLEFT", frame.dropdown.frame, "BOTTOMLEFT", 0, -3)
        contentGroup.frame:SetPoint("BOTTOMRIGHT", frame.buttonGroup.frame, "BOTTOMLEFT", -3, 0)
        contentGroup.frame.widget = contentGroup
        frame.contentGroupRegister[value] = contentGroup
    end

    -- hide old content
    if self.shownContent then
        -- call OnHide register
        if self.onHideList and self.uNameList[self.shownID] and self.onHideList[self.uNameList[self.shownID]] then
            self.onHideList[self.uNameList[self.shownID]](self, self.private[self.shownID], self.shownContent.frame)
        end
        self.shownContent.frame:Hide()
    end

    -- set new frame
    self.shownContent = frame.contentGroupRegister[value]
    self.shownID = value
    self.shownContent.frame:Show()
    if self.executeRegister[value] then
        self.executeRegister[value](self, self.private[value], self.shownContent.frame)
        self.executeRegister[value] = nil
    end
    if self.onShowList and self.uNameList[value] and self.onShowList[self.uNameList[value]] then
        self.onShowList[self.uNameList[value]](self, self.private[value], self.shownContent.frame)
    end
end

local function CreateBaseFrame(frameType)
    local window = AceGUI:Create("Window")
    window:SetTitle("GoldGoblinLoot - "..frameType.name)

    local frame = window.frame
    local content = window.content

    -- some frame settings
    frame:SetMinResize(WIDTH_V,HEIGHT_V)
    frame:SetMaxResize(WIDTH_V+WIDTH_M,HEIGHT_V+HEIGHT_M)
    frame:SetClampedToScreen(true)
    frame:SetToplevel(false)
    frame:SetFrameStrata("MEDIUM")
    --window settings
    window:SetWidth(WIDTH_V)
    window:SetHeight(HEIGHT_V)
    -- remove resizer lines
    window.line1:Hide()
    window.line2:Hide()

    local buttonGroup = AceGUI:Create("SimpleGroup")
    buttonGroup:ClearAllPoints()
    buttonGroup.frame:SetParent(content)
    buttonGroup:SetLayout("List")
    buttonGroup:SetWidth(150)
    buttonGroup.frame:SetPoint("TOPRIGHT", content, 0, 0)
    buttonGroup.frame:SetPoint("BOTTOM", content, 0, 0)
    buttonGroup.obj = frameType
    window.buttonGroup = buttonGroup

    local dropdown = AceGUI:Create("Dropdown")
    dropdown:ClearAllPoints()
    dropdown.frame:SetParent(content)
    dropdown:SetCallback("OnValueChanged", BaseFrameDropDown_OnValueChanged)
    dropdown.frame:SetPoint("TOPLEFT", content, 0, 0)
    dropdown.frame:SetPoint("RIGHT", buttonGroup.frame, "LEFT", -3, 0)
    dropdown.frame:Show()
    dropdown:SetDisabled(true)
    dropdown.obj = frameType
    window.dropdown = dropdown

    window.contentGroupRegister = {}

    GGL.GUI:AddFrameControl("StatisticsFrame-"..frameType.uName, frame, true, false)

    return window
end

local function NpcExportButton_OnClick(self, event, button)
    if GoldGoblinLootDataDB then
        Templates.ShowExportFrame(GoldGoblinLootDataDB["npc_names"], "csv")
    end
end

local BASE_FRAME_PROTO = {
    -- #########
    -- register base

    --- Sets a list for the top dropdown
    -- @param       list        <table> { uName, displayName, guiFillFunc(self, content) }
    ["SetDropDownList"] = function(self, list)
        if not self.frame then
            self.loadDropDownList = list
            return
        end
        local dropdown = self.frame.dropdown
        if not list then
            dropdown:SetDisabled(true)
            dropdown:SetList()
            self.executeRegister = nil
            self.uNameList = nil
            self.private = nil
        else
            dropdown:SetDisabled(false)

            self.uNameList = {}
            self.private = {}
            local fillList, sortList, executeRegister = {}, {}, {}
            for i = 1, #list do
                local entry = list[i]
                sortList[i] = i
                fillList[i] = entry[2]
                if type(entry[3]) == "string" then
                    -- use the function of another entry
                    for j = 1, #list do
                        if list[j][1] == entry[3] then
                            executeRegister[i] = list[j][3]
                            break
                        end
                    end

                else
                    executeRegister[i] = entry[3]
                end
                self.uNameList[entry[1]] = i
                self.uNameList[i] = entry[1]
                self.private[i] = {
                    uName = entry[1],
                    name = entry[2],
                }
            end
            dropdown:SetList(fillList, sortList)
            self.executeRegister = executeRegister
            dropdown:SetValue(1)
            BaseFrameDropDown_OnValueChanged(dropdown, "OnValueChanged", 1)
        end
    end,

    --- Sets onShowlist
    -- @param       list        <table> ["uName"] = function(self, content)
    ["SetOnShowList"] = function(self, list)
        self.onShowList = list
    end,

    --- Sets onHideList
    -- @param       list        <table> ["uName"] = function(self, content)
    ["SetOnHideList"] = function(self, list)
        self.onHideList = list
    end,

    --- Add a new button in the right buttonList
    -- @param       text        <string>
    -- @param       onClickCB   <function>
    -- @param       ...         <strings> Tooltip lines
    ["AddButton"] = function(self, text, onClickCB, ...)
        for i = 1, #self.frame.buttonGroup.children do
            if self.frame.buttonGroup.children.text == text then
                return
            end
        end
        local button = AceGUI:Create("Button")
        button:SetText(text)
        button:SetFullWidth(true)
        button:SetCallback("OnClick", onClickCB)
        GGL.GUI:AddToolTipForAceGUIWidget(button, text, ...)
        self.frame.buttonGroup:AddChild(button)
    end,

    --- Add a npc export button for all npcs
    ["AddNpcExportButton"] = function(self)
        self:AddButton(L["Export NPC Names"], NpcExportButton_OnClick)
    end,

    --- Clear all buttons from the list
    ["ClearButtons"] = function(self)
        self.frame.buttonGroup:ReleaseChildren()
    end,

    --- Opens a specific fram by its unique name
    -- @param       uName       <string> unique name
    ["OpenFrame"] = function(self, uName)
        if not self.uNameList or not self.uNameList[uName] then return end
        self.frame.dropdown:SetValue(self.uNameList[uName])
        BaseFrameDropDown_OnValueChanged(self, "OnValueChanged", self.uNameList[uName])
    end,
    -- register base
    -- #########

    --- Shows the main frame
    ["Show"] = function(self)
        self.frame.frame:Show()
    end,
}

--- Register a new frame type
-- @param       uName       <string> unique name
-- @param       name        <string> display name
-- @returns methods from: BASE_FRAME_PROTO
function Statistics:RegisterFrameType(uName, name)
    if FrameTypeRegister[uName] then return end

    local frameType = {
        uName = uName,
        name = name,
    }

    for k,v in pairs(BASE_FRAME_PROTO) do
        frameType[k] = v
    end

    GGL:AddCallbackHandler(frameType)

    FrameTypeRegister[uName] = frameType

    return frameType, Templates
end

--- Loads the frame of a registered type
-- @param       uName           <string> unique name registered with RegisterFrameType
-- @param       statistics      <core:statistics> statistics table to load
function Statistics:Load(uName, statistics)
    if not uName or not FrameTypeRegister[uName] or not statistics then return end
    local loadType = FrameTypeRegister[uName]

    -- create frame if its not exist
    if not loadType.frame then
        loadType.frame = CreateBaseFrame(loadType)
    end

    -- loaded dropdown list
    if loadType.loadDropDownList then
        loadType:SetDropDownList(loadType.loadDropDownList)
        loadType.loadDropDownList = nil
    end

    -- set statistics
    loadType.statistics = statistics

    -- show frame
    loadType.frame:Show()

    return loadType
end


function _G.GGL_TEST()
    Statistics:Load("baseFrame", {})
end