local AddonName, GGL = ...
local GUI = GGL.GUI
local Sets = GGL.Options.Sets
local Global = {}
GGL.Options.Sets_Zone = Global

local L = GGL.Locale

-- lua
local type = type
local ipairs, pairs = ipairs, pairs
local strmatch, strformat = string.match, string.format

-- WoW
local GetMapInfo = C_Map.GetMapInfo
local EJ_GetInstanceInfo = EJ_GetInstanceInfo
local GetDifficultyInfo = GetDifficultyInfo

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

local CONTINENT_MAP_LIST = GGL.CONTINENT_MAP_LIST
local INSTANCE_MAP_LIST, EXPANSION_NAME = GGL.INSTANCE_MAP_LIST, GGL.EXPANSION_NAME
local INSTANCE_ID_DATA  = GGL.INSTANCE_ID_DATA
local MAP_ID_DATA = GGL.MAP_ID_DATA

-- ##################
-- local
-- ##################
local Helper

local ID_AND_DIFF_MATCH = "(%d+):(%d+)"
local NAME_AND_DIFF_FORMAT = "%s (%s)"
-- ##################
-- local helper
-- ##################
local function SaveDB()
    local t = Helper.Data["selectedDB"]
    if t.e then
        for k,v in pairs(t.e) do
            if v and type(v) == "number" and v < 1 then
                t.e[k] = nil
            end
        end
    end
    Helper:SaveDB()
end

-- ##################
-- options
-- ##################
local function ClearExpansionData(db, mapList, isRaid)
    local source = isRaid and mapList.raid or mapList.dungeon
    for instanceIndex, instanceID in ipairs(source) do
        db[instanceID] = nil
    end
    if not isRaid then
        ClearExpansionData(db, mapList, true)
    end
end

local function EnableButton_OnValueChanged(self, event, state)
    local data = Helper.Data
    local db = data["selectedDB"]
    local expansionID = data["selectedTabID"]

    if state == true then
        ClearExpansionData(db, INSTANCE_MAP_LIST[expansionID])
        if db.e then
            db.e[expansionID] = nil
            if not next(db.e) then
                db.e = nil
            end
        end
        SaveDB()
    elseif state == false then
        if not db.e then db.e = {} end
        ClearExpansionData(db, INSTANCE_MAP_LIST[expansionID])
        db.e[expansionID] = true
        SaveDB()
    else -- nil
        if not db.e then db.e = {} end
        db.e[expansionID] = 0
    end

    Helper:RefreshSelectedTab()
end

local function InstanceCheck_OnValueChanged(self, event, state)
    local zoneID = self:GetUserData("zoneID")
    local data = Helper.Data
    local db = data["selectedDB"]
    local expansionID = data["selectedTabID"]
    if state then
        db[zoneID] = true
        db.e[expansionID] = db.e[expansionID] + 1
    else
        db[zoneID] = nil
        db.e[expansionID] = db.e[expansionID] - 1
    end
    SaveDB()
end

local function AddCheckButton(db, content, zoneID, name)
    local check = AceGUI:Create("CheckBox")
    check:SetLabel(name)
    check:SetValue(db[zoneID])
    check:SetCallback("OnValueChanged", InstanceCheck_OnValueChanged)
    check:SetUserData("zoneID", zoneID)
    content:AddChild(check)
end

local function CreateCheckButtons_Instance(source, db, content)
    for instanceIndex, instanceID in ipairs(source) do
        if type(instanceID) == "string" then
            local instanceIDNew, difficultyID = strmatch(instanceID, ID_AND_DIFF_MATCH)
            local name = EJ_GetInstanceInfo(instanceIDNew)
            AddCheckButton(db, content, tonumber(instanceIDNew), strformat(NAME_AND_DIFF_FORMAT, name, GetDifficultyInfo(tonumber(difficultyID))))
        else
            local name = EJ_GetInstanceInfo(instanceID)
            AddCheckButton(db, content, instanceID, name)
        end
    end
end

local function Instance_SetOnTabGroupSelected(content, db, tabID)
    -- top part
    local base = AceGUI:Create("SimpleGroup")
    base:SetFullWidth(true)
    base:SetLayout("Flow")
    content:AddChild(base)

    local enableCheck = AceGUI:Create("CheckBox")
    enableCheck:SetTriState(true) -- true, nil, false
    if db.e and db.e[tabID] == true then
        enableCheck:SetValue(false)
        enableCheck:SetLabel(L["|cffff0000Disabled|r"])
    elseif db.e and type(db.e[tabID]) == "number" then
        enableCheck:SetValue(nil)
        enableCheck:SetLabel(L["|cff00ff00Enabled|r"])
    else
        enableCheck:SetValue(true)
        enableCheck:SetLabel(L["|cff00ff00Enabled|r"])
    end
    enableCheck:SetCallback("OnValueChanged", EnableButton_OnValueChanged)
    base:AddChild(enableCheck)

    -- disabled skip from now
    if enableCheck:GetValue() == false then return end

    -- on nil tristate
    if enableCheck:GetValue() == nil then
        local contDungeon = AceGUI:Create("SimpleGroup")
        contDungeon:SetFullWidth(true)
        contDungeon:SetLayout("Flow")

        local contRaid = AceGUI:Create("SimpleGroup")
        contRaid:SetFullWidth(true)
        contRaid:SetLayout("Flow")

        CreateCheckButtons_Instance(INSTANCE_MAP_LIST[tabID].dungeon, db, contDungeon)
        CreateCheckButtons_Instance(INSTANCE_MAP_LIST[tabID].raid, db, contRaid)

        content:AddChild(contRaid)
        content:AddChild(contDungeon)
    end
end

local function Instance_SetOnTabGroupConfigSelected(content, db, tabID)

end

local function Instance_Options(AceGUI, container)
    Helper = Sets:GetBaseGroup(container, GGL.SET_ID_LIST.Instance)

    local tabs = {}

    for i = 0, #INSTANCE_MAP_LIST do
        tabs[i+1] = {
            value = i,
            text = EXPANSION_NAME[i]
        }
    end

    Helper:SetOnTabGroupSelected(Instance_SetOnTabGroupSelected)
    Helper:SetOnTabGroupConfigSelected(Instance_SetOnTabGroupConfigSelected)

    Helper:SetTabGroupContent(tabs, true)
end

-- Add into Options
GGL.Options:AddIntoTree(Instance_Options, L["Instances"], "sets_instances", "sets")

--[[
GGL.INSTANCE_MAP_LIST = {
	[1] = { -- classic
        dungeon={221,242,291,235,226,250,280,213,300,301,431,302,476,310,317,225,220,230,279,219},
        raid={232,287,247,319},
	},
	[2] = { -- bc
        dungeon={256,347,348,272,274,258,260,269,273,261,266,267,246,265,263,262},
        raid={350,330,331,332,334,329,339,335},
	},
}
]]