local AddonName, GGL = ...
local GUI = GGL.GUI
local Global = {}
GGL.Options.Settings = Global

local L = GGL.Locale

-- lua

-- WoW

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

-- ##################
-- local
-- ##################
local function WidgetOnRelease(self, event)

end


-- ##################
-- Settings
-- ##################

local function Name_OnEnterPressed(self, event, text)
    text = GGL.FormatDisplayName(text)
    GGL.db.Settings.Main.displayName = text
    GGL.Templates.BNWoWPlayer:GetSelfPlayer():SetDisplayName(text)
    self:SetText(text)
    self:ClearFocus()
end

local function ShowMinimapButton_OnValueChanged(self, event, state)
    GGL:MMB_SetShownState(state)
end

local function GglAlertAnchor_OnClick(self, event, mb)
    GGL.LootAlert:UnlockAlertFrameAnchor(nil)
end

local function AutoLootCheck_OnValueChanged(self, event, state)
    GGL.db.Settings.FastAutoLoot = state
end

local function ShowDestroyGrayButton_OnValueChanged(self, event, state)
    GGL.db.Settings.ShowDestroyGrayButton = state
    if GGL.GUI.MainWindow.frame then
        GGL.GUI.MainWindow.frame:RefreshDBSettings()
    end
end

local function Settings_Options(AceGUI, container)
    local lines = {}
    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow")
    scroll:SetCallback("OnRelease", WidgetOnRelease)
    container:AddChild(scroll)

    local name = AceGUI:Create("EditBox")
    name:SetFullWidth(true)
    name:SetLabel(L["Name"])
    name:SetText(GGL.Templates.BNWoWPlayer:GetSelfPlayer().displayName)
    name:SetMaxLetters(GGL.MAX_NAME_LENGTH)
    name:SetCallback("OnEnterPressed", Name_OnEnterPressed)
    scroll:AddChild(name)

    lines[#lines+1] = AceGUI:Create("Heading")
    lines[#lines]:SetText(L["Minimap Button"])
    lines[#lines]:SetFullWidth(true)
    scroll:AddChild(lines[#lines])

    local showMinimapButton = AceGUI:Create("CheckBox")
    showMinimapButton:SetFullWidth(true)
    --:SetRelativeWidth(0.5)
    showMinimapButton:SetLabel(SHOW)
    showMinimapButton:SetValue(not GGL.db.MinimapButton.hide)
    showMinimapButton:SetCallback("OnValueChanged", ShowMinimapButton_OnValueChanged)
    scroll:AddChild(showMinimapButton)

    lines[#lines+1] = AceGUI:Create("Heading")
    lines[#lines]:SetText(L["Destroy gray items button"])
    lines[#lines]:SetFullWidth(true)
    scroll:AddChild(lines[#lines])

    local showDestroyGreyButtonCheck = AceGUI:Create("CheckBox")
    showDestroyGreyButtonCheck:SetFullWidth(true)
    showDestroyGreyButtonCheck:SetLabel(SHOW)
    showDestroyGreyButtonCheck:SetValue(GGL.db.Settings.ShowDestroyGrayButton)
    showDestroyGreyButtonCheck:SetCallback("OnValueChanged", ShowDestroyGrayButton_OnValueChanged)
    scroll:AddChild(showDestroyGreyButtonCheck)

    lines[#lines+1] = AceGUI:Create("Heading")
    lines[#lines]:SetText(L["GGL Loot Alert Anchor"])
    lines[#lines]:SetFullWidth(true)
    scroll:AddChild(lines[#lines])

    local gglAlertAnchor = AceGUI:Create("Button")
    gglAlertAnchor:SetText(L["Show / Hide"])
    gglAlertAnchor:SetFullWidth(true)
    gglAlertAnchor:SetCallback("OnClick", GglAlertAnchor_OnClick)
    scroll:AddChild(gglAlertAnchor)

    lines[#lines+1] = AceGUI:Create("Heading")
    lines[#lines]:SetText(L["Fast Loot"])
    lines[#lines]:SetFullWidth(true)
    scroll:AddChild(lines[#lines])

    local autoLootCheck = AceGUI:Create("CheckBox")
    autoLootCheck:SetFullWidth(true)
    autoLootCheck:SetLabel(ENABLE)
    autoLootCheck:SetValue(GGL.db.Settings.FastAutoLoot)
    autoLootCheck:SetCallback("OnValueChanged", AutoLootCheck_OnValueChanged)
    scroll:AddChild(autoLootCheck)

    -- resize scrollFrame
    scroll:DoLayout()
end

-- Add into Options
GGL.Options:AddIntoTree(Settings_Options, L["General Settings"], "settings")

-- ##################
-- Settings - Streamer header
-- ##################
local StreamerDbHandler

local function Streamer_BattleTag_OnValueChanged(self, event, text)
    if not text or text == "" then
        local _, battleTag = BNGetInfo()
        StreamerDbHandler:Set("battleTag", nil)
        self:SetText(battleTag)
    else
        StreamerDbHandler:Set("battleTag", text)
    end
    self:ClearFocus()
end

local function Streamer_Title_OnValueChanged(self, event, text)
    StreamerDbHandler:Set("title", text)
    self:ClearFocus()
end

local function Streamer_Check_OnValueChanged(self, event, state)
    StreamerDbHandler:Set(self:GetUserData("key"), state)
end

local function Settings_Streamer(AceGUI, container)
    StreamerDbHandler = StreamerDbHandler or GGL.Options:GetDbGetterAndSetter(GGL.db, "Settings", "Streamer")

    local _, battleTag = BNGetInfo()

    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow")
    scroll:SetCallback("OnRelease", WidgetOnRelease)
    container:AddChild(scroll)

    local enabled = AceGUI:Create("CheckBox")
    enabled:SetFullWidth(true)
    enabled:SetLabel(ENABLE)
    enabled:SetValue(StreamerDbHandler:Get("enabled"))
    enabled:SetCallback("OnValueChanged", Streamer_Check_OnValueChanged)
    enabled:SetUserData("key", "enabled")
    scroll:AddChild(enabled)

    --[[
    local hideBanner = AceGUI:Create("CheckBox")
    hideBanner:SetFullWidth(true)
    hideBanner:SetLabel("Hide banner")
    hideBanner:SetValue(StreamerDbHandler:Get("hideBanner"))
    hideBanner:SetCallback("OnValueChanged", Streamer_Check_OnValueChanged)
    hideBanner:SetUserData("key", "hideBanner")
    scroll:AddChild(hideBanner)
    ]]

    local title = AceGUI:Create("EditBox")
    title:SetFullWidth(true)
    title:SetLabel(L["Title"])
    title:SetText(StreamerDbHandler:Get("title"))
    title:SetCallback("OnEnterPressed", Streamer_Title_OnValueChanged)
    scroll:AddChild(title)

    local battleTagEB = AceGUI:Create("EditBox")
    battleTagEB:SetFullWidth(true)
    battleTagEB:SetLabel(L["Battle tag"])
    battleTagEB:SetText(StreamerDbHandler:Get("battleTag") or battleTag)
    battleTagEB:SetCallback("OnEnterPressed", Streamer_BattleTag_OnValueChanged)
    scroll:AddChild(battleTagEB)
end

-- Add into Options
--GGL.Options:AddIntoTree(Settings_Streamer, L["Streamer"], "settings_streamer", "settings")