local AddonName, GGL = ...
local Templates = GGL.Templates
local Sender = GGL.Sender
local Session = Templates.Session
local SyncStatic = Templates.Sync

local Challenge, ChallengeHelper = Templates:Register("Challenge", "Session", "Sync")

-- lua
local sub, match = string.sub, string.match

-- wow
local C_TimerAfter = C_Timer.After

-- ##################
-- local
-- ##################
local SessionSave = {}

local INVITE_STATES = GGL.INVITE_KEY_LIST

local PLAYER_STATUS_UPDATE_SEND = "%d\002%d\002%s\002%s\002%s\002%s\002%s\002%s\002%s" -- rank, attendees, curGold, upGold, upName, downGold, downName, firstGold, firstName
local PLAYER_STATUS_UPDATE_GET = "^(%d*)\002(%d*)\002(.*)\002(.*)\002(.*)\002(.*)\002(.*)\002(.*)\002(.*)"

-- set timers only one time
local TimerRegister = {}
local function Set_C_TimerAfter(after, method)
    if not TimerRegister[method] then
        TimerRegister[method] = true
        C_TimerAfter(after, method)
    end
end

-- ##################
-- static
-- ##################
local Static = {}
Templates.Challenge = Static

function Static:GetSession(sessionDB, sessionID)
    sessionDB = sessionDB or GGL.sessionDb.Sessions
    if sessionID and SessionSave[sessionID] then
        return SessionSave[sessionID], false
    else
        return Templates:New("Challenge", sessionDB, sessionID), true
    end
end

-- ##################
-- template
-- ##################
Challenge.sessionFormat = "C%d"
Challenge.sessionType = "C"

local function CB_HostStartsSession(self, session, duration, endTime, settingsString)
    session:SetData()
    session:SetSettingsString(settingsString)
    if not endTime or endTime == 0 then
        -- duration for endless is the startTime send from Host
        session.db.timeElapsed = GGL.GetTime() - duration
        session:SetTimerEndless()
    else
        session:SetTimerData(endTime - duration, endTime)
    end
    session:Start()
end

local function CB_HostEndsSession(self, session)
    session:Close()
end

local function OnPlayerStatusUpdate(self, prefix, sender, message, isBNMessage)
    local session = SessionSave[sub(prefix, 1, -2)]
    if not session then return end

    -- get playerTemplate
    sender = SyncStatic.GetPlayerFromSync(sender)
    if not sender or session.inviteTemplate:GetHost() ~= sender then return end

    local rank, attendees, curGold, upGold, upName, downGold, downName, firstGold, firstName = match(message, PLAYER_STATUS_UPDATE_GET)
    if not rank then return end

    session:UpdateOwnData(rank, attendees, tonumber(curGold), tonumber(upGold), upName, tonumber(downGold), downName, tonumber(firstGold), firstName)
end

-- ## Instance reset
local function OnPlayerInstanceResetUpdate(self, prefix, sender, message, isBNMessage)
    local session = SessionSave[sub(prefix, 1, -3)]
    if not session then return end

    -- get playerTemplate
    sender = SyncStatic.GetPlayerFromSync(sender)
    if not sender then return end

    local pass, data = GGL:Deserialize(message)
    if not pass or not data then return end
    session:UpdateResetData(sender, data)
end
-- ## Instance reset end

function Challenge:Constructor(sessionDB, sessionID)
    if sessionID and SessionSave[sessionID] then
        return SessionSave[sessionID]
    end
    -- base
    self:CallBaseFunction("Session", "Constructor", sessionDB, sessionID)
    self:CallBaseFunction("Sync", "Constructor")

    -- sync settings with players
    self:SyncSettingsWithPlayers(true)

    self.inviteTemplate:RegisterCallbackEvent("HostStartsSession", CB_HostStartsSession)
    self.inviteTemplate:RegisterCallbackEvent("HostEndsSession", CB_HostEndsSession)

    SessionSave[self.db.sessionID] = self
end

function Challenge:UpdateInstanceReseteList()
    local oldData = self.db.instanceResetData
    local newData = {}

    for playerMergeID, playerData in pairs(oldData) do
        local mergeID = self:GetPlayerMergeID(SyncStatic.GetPlayerFromSync(playerMergeID))
        if mergeID and (not newData[mergeID] or newData[mergeID][3] < playerData[3]) then
            newData[mergeID] = playerData
        end
    end

    self.db.instanceResetData = newData
end

function Challenge:GetInstanceResetData()
    return self.db.instanceResetData
end

function Challenge:GetInstanceResetDataForPlayer(playerTemplate)
    if not playerTemplate then return end
    return self.db.instanceResetData[self:GetPlayerMergeID(playerTemplate)]
end

local function CB_CFrameOnNewItem(self, itemID, itemString, zoneID, sourceID, ammount, copperValue, mainSourceFlag, subSourceFlag, sourcePlayer)
    if self.challengeFrame then
        self.challengeFrame:SetPlayerGold(sourcePlayer, copperValue * ammount, false)
    end
    if self.topItemsFrame then
        self.topItemsFrame:AddItem(sourcePlayer, itemID, self:LoggedItemUseVariations(itemID, itemString) and itemString or nil, ammount, copperValue, zoneID, sourceID, mainSourceFlag, subSourceFlag)
    end
end

local function CB_CFrameOnAddMoney(self, copper, sourcePlayer)
    if not self.challengeFrame then return end
    --self.challengeFrame:SetGoldCount(copper, true)
    self.challengeFrame:SetPlayerGold(sourcePlayer, copper, false)
end

local function CB_CFrameOnNewSyncLoot(self, sourcePlayer, lootTime, zoneIniID, difficultyID, itemID, itemLog, ammount, sourceID, mainSourceFlag, subSourceFlag)
    self.pauseLog = true
    if itemID == 0 then
        self:AddMoney(ammount, nil, sourceID, mainSourceFlag, subSourceFlag, nil, zoneIniID, difficultyID, lootTime, nil, sourcePlayer)
    else
        self:AddNewItem(itemID, itemLog, ammount, nil, sourceID, mainSourceFlag, subSourceFlag, nil, zoneIniID, difficultyID, lootTime, nil, sourcePlayer)
    end
    self.pauseLog = nil
end

local function CB_CFrameOnPlayerAccepted(self, player)
    if self.challengeFrame then
        self.challengeFrame:AddPlayer(player)
        self.challengeFrame:SetPlayerInviteState(player, INVITE_STATES.ACCEPTED)
    end
    if self.topItemsFrame then
        self.topItemsFrame:SetPlayerInviteState(player, INVITE_STATES.ACCEPTED)
    end
end

local function CB_CFrameOnPlayerDeclined(self, player)
    if self.challengeFrame then
        self.challengeFrame:SetPlayerInviteState(player, INVITE_STATES.DECLINED)
    end
    if self.topItemsFrame then
        self.topItemsFrame:SetPlayerInviteState(player, INVITE_STATES.DECLINED)
    end
end

local function CB_CFrameOnPlayerInvited(self, player)
    if self.challengeFrame then
        self.challengeFrame:SetPlayerInviteState(player, INVITE_STATES.INVITED)
    end
    if self.topItemsFrame then
        self.topItemsFrame:SetPlayerInviteState(player, INVITE_STATES.INVITED)
    end
end

local function CB_CFrameOnPlayerMergeIDUpdate(self)
    if self.challengeFrame then
        self.challengeFrame:UpdateMergeList()
    end
    if self.topItemsFrame then
        self.topItemsFrame:UpdateMergeList()
    end
    self:UpdateInstanceReseteList()
end

local function CB_SetDataRefresh(self)
    if self.topItemsFrame then
        self:RemoveFromTopItemsFrame()
        self:AddIntoTopItemsFrame(true)
    end
    if self.challengeFrame then
        self:RemoveFromChallengeFrame()
        self:AddIntoChallengeFrame(true)
    end

    self:RebuildFromLog()
end

-- #################
-- ChallengeFrame
-- #################
function Challenge:RemoveFromChallengeFrame()
    if not self.challengeFrame then return end
    local frame = self.challengeFrame

    frame.timer:FinishTimer()

    frame.runningSession = nil

    frame:SetUpdateInterval(nil)

    -- remove from minimap button
    GGL.GUI.Challenge:RemoveFromMiniMapButton()

    -- one last update
    frame:UpdateList()

    -- freeze the list
    frame:UnregisterPlayerUpdates()

    -- show the export button
    frame:ShowExportButton(true)

    -- callback
    self:UnregisterCallbackEvent("AddItem", CB_CFrameOnNewItem)
    self:UnregisterCallbackEvent("AddMoney", CB_CFrameOnAddMoney)
    self:UnregisterCallbackEvent("NewSyncLoot", CB_CFrameOnNewSyncLoot)
    self:UnregisterCallbackEvent("PlayerAccepted", CB_CFrameOnPlayerAccepted)
    self:UnregisterCallbackEvent("PlayerDeclined" , CB_CFrameOnPlayerDeclined)
    self:UnregisterCallbackEvent("PlayerInvited" , CB_CFrameOnPlayerInvited)
    self:UnregisterCallbackEvent("PlayerMergeIDUpdate" , CB_CFrameOnPlayerMergeIDUpdate)

    self.challengeFrame = nil
end

function Challenge:AddIntoChallengeFrame(add)
    if not self.db or self.db.isClosed or not self:PlayerIsHost() then return end
    if add then
        local frame = GGL.GUI.Challenge:CreateFrame()

        -- start timer if its a running session or just set timer
        local db = self.db
        if self.db.endlessTime  then
            if db.isRunning  then
                frame.timer:SetEndless(db.timeElapsed)
                frame.timer:StartTimer()
                frame:Show()
            end
        elseif db.isRunning then
            frame.timer:SetTime(db.startTime, db.endTime)
            frame.timer:IsCounter(false)
            frame.timer:StartTimer()
        elseif db.durationTime then
            frame.timer:SetDuration(db.durationTime)
        end

        -- set data
        if self.challengeFrame ~= frame then
            self.challengeFrame = frame

            -- clear old frame
            frame:Clear()
            self.rebuildSync = nil

            -- set session for challenge frame
            frame:SetSession(self)

            -- first fill with lootLog data
            for playerSyncID in pairs(self.db.lootLog) do
                if playerSyncID ~= self.selfDbKey then
                    local playerTemplateLootLog = Templates.Sync.GetPlayerFromSync(playerSyncID, true)
                    frame:AddPlayer(playerTemplateLootLog)
                end
            end
            -- set chars
            frame:AddPlayer(self.selfPlayer)
            local playerList = self.playerList
            for player, inviteState in self:IterInvitedPlayers() do
                if (inviteState == INVITE_STATES.ACCEPTED) or (self.db and self.db.lootLog and self.db.lootLog[self:GetPlayerSyncID(player)]) then
                    frame:AddPlayer(player)
                end
            end

            -- update
            frame:SetUpdateInterval(15)

            -- add into minimap button
            GGL.GUI.Challenge:AddIntoMiniMapButton()

            -- show the export button
            frame:ShowExportButton(false)

            -- callback
            self:RegisterCallbackEvent("AddItem", CB_CFrameOnNewItem)
            self:RegisterCallbackEvent("AddMoney", CB_CFrameOnAddMoney)
            self:RegisterCallbackEvent("NewSyncLoot", CB_CFrameOnNewSyncLoot)
            self:RegisterCallbackEvent("PlayerAccepted", CB_CFrameOnPlayerAccepted)
            self:RegisterCallbackEvent("PlayerDeclined" , CB_CFrameOnPlayerDeclined)
            self:RegisterCallbackEvent("PlayerInvited" , CB_CFrameOnPlayerInvited)
            self:RegisterCallbackEvent("PlayerMergeIDUpdate" , CB_CFrameOnPlayerMergeIDUpdate)

            -- update
            frame:UpdateMergeList()
        end
    else
        self:RemoveFromChallengeFrame()
    end
end

-- #################
-- SmallChallengeFrame
-- #################
function Challenge:RemoveFromSmallChallengeFrame()
    if not self.smallChallengeFrame then return end
    local frame = self.smallChallengeFrame

    frame.timer:FinishTimer()

    frame.runningSession = nil

    -- remove from minimap button
    GGL.GUI.Challenge:RemoveSmallFromMiniMapButton()

    self.smallChallengeFrame = nil
end

function Challenge:AddIntoSmallChallengeFrame(add)
    if not self.db or self.db.isClosed then return end

    local frame = GGL.GUI.Challenge:ShowSmallFrame()

    -- start timer if its a running session or just set timer
    local db = self.db
    if self.db.endlessTime  then
        if db.isRunning  then
            frame.timer:SetEndless(db.timeElapsed)
            frame.timer:StartTimer()
            frame:Show()
        end
    elseif db.isRunning then
        frame.timer:SetTime(db.startTime, db.endTime)
        frame.timer:IsCounter(false)
        frame.timer:StartTimer()
    elseif db.durationTime then
        frame.timer:SetDuration(db.durationTime)
    end

    -- clear all
    frame:Clear()

    -- add into small frame
    GGL.GUI.Challenge:AddSmallIntoMiniMapButton()

    self.smallChallengeFrame = frame
end

-- #################
-- TopItems
-- #################
function Challenge:RemoveFromTopItemsFrame()
    if not self.topItemsFrame then return end
    local frame = self.topItemsFrame

    -- Unregister all player events
    frame:UnregisterAllPlayerEvents()

    -- remove from minimap button
    GGL.GUI.TopItems:RemoveFromMiniMapButton()

    self.topItemsFrame = nil
end

function Challenge:AddIntoTopItemsFrame(add)
    if not self.db or self.db.isClosed then return end

    local frame = GGL.GUI.TopItems:Show()

    -- clear all
    frame:Clear()

    -- set session
    frame:SetSession(self)

    -- update
    frame:UpdateMergeList()

    -- add into small frame
    GGL.GUI.TopItems:AddIntoMiniMapButton()

    self.topItemsFrame = frame
end

-- #################
-- Challenge
-- #################
function Challenge:Start()
    -- always show main and challenge frame
    self:AddIntoMainFrame(true)
    if self:PlayerIsHost() then
        self:AddIntoChallengeFrame(true)
        self:AddIntoTopItemsFrame(true)
    end

    self:CallBaseFunction("Session", "Start")
    self:CallBaseFunction("Sync", "Start")
    if not self.db.isRunning then return end

    if self:PlayerIsHost() then
        self:AddIntoChallengeFrame(true)
        GGL.GUI.Challenge:Show()

        if not self.__playerStatusUpdate then
            self.__playerStatusUpdate = function()
                if not self.db.isRunning or not self.challengeFrame then
                    if TimerRegister and self.__playerStatusUpdate then
                        TimerRegister[self.__playerStatusUpdate] = nil
                    end
                    self.__playerStatusUpdate = nil
                    return
                end

                local challengeChildren = self.challengeFrame.charContainer.children
                local firstName, firstGold
                if challengeChildren and challengeChildren[1] then
                    firstName = challengeChildren[1]:GetPlayer().displayName or ""
                    firstGold = tostring(challengeChildren[1]:GetMoney() or 0)
                end
                for i = 1, #challengeChildren do
                    local childPrev = challengeChildren[i-1]
                    local child = challengeChildren[i]
                    local childNext = challengeChildren[i+1]
                    if child:GetPlayer() == self.selfPlayer then
                        self:UpdateOwnData(
                            child:GetRank(),
                            #challengeChildren,
                            child:GetMoney(),
                            childPrev and childPrev:GetMoney() or 0,
                            childPrev and childPrev:GetPlayer().displayName or "",
                            childNext and childNext:GetMoney() or 0,
                            childNext and childNext:GetPlayer().displayName or "",
                            tonumber(firstGold),
                            firstName
                        )
                    else
                        if self.inviteTemplate:GetPlayerInviteState(child:GetPlayer()) == INVITE_STATES.ACCEPTED then
                            -- rank, attendees, curGold, upGold, upName, downGold, downName
                            self.senderPlayerStatusUpdate:SendMessage(child:GetPlayer(),
                                format(PLAYER_STATUS_UPDATE_SEND,
                                    child:GetRank(),
                                    #challengeChildren,
                                    tostring(child:GetMoney()),
                                    childPrev and tostring(childPrev:GetMoney()) or "0",
                                    childPrev and childPrev:GetPlayer().displayName or "",
                                    childNext and tostring(childNext:GetMoney()) or "0",
                                    childNext and childNext:GetPlayer().displayName or "",
                                    firstGold,
                                    firstName
                                )
                            )
                        end
                    end
                end

                C_TimerAfter((#challengeChildren*0.1) + 15, self.__playerStatusUpdate)
            end
            Set_C_TimerAfter(16, self.__playerStatusUpdate)
        end
    end

    -- show small frame
    self:AddIntoSmallChallengeFrame(true)

    -- SetDataRefresh
    self:RegisterCallbackEvent("SetDataRefresh", CB_SetDataRefresh)

    -- Instance resets
    if not self.challenge_cbOnInstancesResetUpdate then
        self.challenge_cbOnInstancesResetUpdate = function(session, data)
            self:SendResetDataToHost()
        end
        self:RegisterCallbackEvent("OnInstancesResetUpdate", self.challenge_cbOnInstancesResetUpdate)
    end

    -- sender
    if not self.senderPlayerStatusUpdate or not self.senderPlayerStatusUpdate.Unregister then
        self.senderPlayerStatusUpdate = GGL.Sender:RegisterReceiver(self.db.sessionID.."u", OnPlayerStatusUpdate)
    end
    if not self.senderInstanceResets or not self.senderInstanceResets.Unregister then
        self.senderInstanceResets = GGL.Sender:RegisterReceiver(self.db.sessionID.."ir", OnPlayerInstanceResetUpdate)
    end

    -- update
    if self.topItemsFrame then
        self.topItemsFrame:UpdateMergeList()
    end

    -- send reset data
    self:SendResetDataToHost()

    return self.db.startTime, self.db.endTime, self.db.durationTime
end

function Challenge:SetData(mainSet, setRefresh)
    self:CallBaseFunction("Session", "SetData", mainSet, setRefresh)

    if self.challengeFrame then
        self:AddIntoChallengeFrame(true)
    end
end

function Challenge:SendResetDataToHost(data)
    if self:PlayerIsHost() then return end

    data = data or self:UpdateOwnResetData(true)

    self.senderInstanceResets:SendMessage(self:GetHostPlayer(), GGL:Serialize(data))
end

--- Updates data for the small frame
-- @param       rank        <number> current rank
-- @param       attendees   <number> attendees in this session
-- @param       curGold     <number> (copper) current player money value
-- @param       upGold      <number> (copper) current player money for the rank over the player
-- @param       upName      <string> name of the player for the rank over the player
-- @param       downGold    <number> (copper) current player money for the rank under the player
-- @param       downName    <string> name of the player for the rank under the player
-- @param       firstGold    <number> (copper) first player money
-- @param       downName    <string> first player name
function Challenge:UpdateOwnData(rank, attendees, curGold, upGold, upName, downGold, downName, firstGold, firstName)
    if self.smallChallengeFrame then
        local smallFrame = self.smallChallengeFrame

        smallFrame:SetFirstPlayerData(firstGold, firstName, attendees)
        smallFrame:SetUpPlayerData(upGold, upName, rank-1)
        smallFrame:SetPlayerData(curGold, self.selfPlayer.displayName, rank)
        smallFrame:SetDownPlayerData(downGold, downName, rank+1)
    end
end

local function UnregisterAll(self)
    self:RemoveFromChallengeFrame()
    self:RemoveFromSmallChallengeFrame()

    if self.senderPlayerStatusUpdate and self.senderPlayerStatusUpdate.Unregister then
        self.senderPlayerStatusUpdate:Unregister()
    end
    if self.senderInstanceResets and self.senderInstanceResets.Unregister then
        self.senderInstanceResets:Unregister()
    end

    SessionSave[self.db.sessionID] = nil
end

function Challenge:Unregister()
    self:CallBaseFunction("Session", "Unregister")
    self:CallBaseFunction("Sync", "Unregister")

    UnregisterAll(self)
end

function Challenge:Stop()
    self:CallBaseFunction("Session", "Stop")
    self:CallBaseFunction("Sync", "Stop")

    UnregisterAll(self)

    -- keep open
    self.inviteTemplate:RegisterCallbackEvent("HostStartsSession", CB_HostStartsSession)
    self.inviteTemplate:RegisterCallbackEvent("HostEndsSession", CB_HostEndsSession)
    SessionSave[self.db.sessionID] = self
end

--- Close the session
function Challenge:Close()
    -- send one last status update
    if self.__playerStatusUpdate then
        self.__playerStatusUpdate()
    end
    self:CallBaseFunction("Session", "Close")
    self:CallBaseFunction("Sync", "Close")

    UnregisterAll(self)
end